@@ -4,8 +4,8 @@
 """Module for the main application classes."""
 from pathlib import Path
 
-from PyQt6.QtCore import *
-from PyQt6.QtWidgets import *
+from PySide6.QtCore import *
+from PySide6.QtWidgets import *
 from toml.decoder import CommentValue
 
 from ..tomlfile import *
@@ -32,14 +32,14 @@ class Theme:
 
 
 class GetterApp(QApplication):
-    """The main HaloInfiniteGetter PyQt application that runs in the background and manages the process."""
+    """The main HaloInfiniteGetter PySide application that runs in the background and manages the process."""
     _legacy_style: str = None
 
     def __init__(self, argv: list[str], settings: TomlFile) -> None:
         """Create a new app with the given arguments and settings."""
         super().__init__(argv)
         self.settings:        TomlFile = settings
-        self.themes:          dict[str, Theme] = {}
+        self.themes:          dict[str, Theme] = {}  # PyCharm detects this as a dict[str, str], despite explicit type hints.
         self.theme_index_map: dict[str, int] = {}
 
         self.settings.hook_event('$reload', self.load_themes)
@@ -56,7 +56,7 @@ class GetterApp(QApplication):
         """
         if self._legacy_style is None:
             self.__class__._legacy_style = self.styleSheet()
-        self.themes['legacy'] = Theme('legacy', self._legacy_style, 'Legacy (Default PyQt)')
+        self.themes['legacy'] = Theme('legacy', self._legacy_style, 'Legacy (Default Qt)')
 
         for id_, theme in self.settings['gui/themes'].items():
             if not isinstance(theme, dict):
@@ -76,11 +76,12 @@ class GetterApp(QApplication):
                 self.themes[id_] = Theme(**theme)
                 file.close()
 
+        # noinspection PyUnresolvedReferences
         self.theme_index_map = {theme_id: i for i, theme_id in enumerate(theme.id for theme in self.sorted_themes)}
         self.update_stylesheet()
 
     @property
     def sorted_themes(self) -> list[Theme]:
         """List of themes sorted by their display name."""
-        # FIXME: Doesn't crash, but also doesn't sort by display_name
+        # noinspection PyTypeChecker
         return sorted(self.themes.values(), key=lambda theme: theme.display_name)
