@@ -7,14 +7,15 @@ import webbrowser
 from collections.abc import Callable
 from pathlib import Path
 
-from PyQt6.QtCore import *
-from PyQt6.QtGui import *
-from PyQt6.QtWidgets import *
+from PySide6.QtCore import *
+from PySide6.QtGui import *
+from PySide6.QtWidgets import *
 
 from .._version import __version__
 from ..client import Client
 from ..constants import *
 from ..tomlfile import *
+from ..utils import HTTP_CODE_MAP
 from .app import *
 from .menus import *
 
@@ -40,8 +41,10 @@ class SettingsWindow(QWidget):
         self.resize(size)
         self.setFixedWidth(self.width())
         self.settings.hook_event('$fail:import', lambda: QMessageBox.warning(
-            self, 'Import Error', f'Could not import settings from "{self.settings.path}". Make sure both the file and '
-                                  f'the parent folder exist, and that the file contains valid TOML.'))
+            self, 'Import Error',
+            f'Could not import settings from "{self.settings.path}". Make sure both the file and '
+            f'the parent folder exist, and that the file contains valid TOML.'
+        ))
 
         self.save_button:             QPushButton
         self.theme_dropdown:          QComboBox
@@ -53,23 +56,28 @@ class SettingsWindow(QWidget):
         self._init_ui()
 
     def _init_ui(self) -> None:
-        self.save_button = QPushButton('Save', clicked=self.save_settings)
-        reload_button = QPushButton('Reload', clicked=self.reload_settings)
-        import_button = QPushButton('Import Settings', clicked=self.import_settings)
-        export_button = QPushButton('Export Settings', clicked=self.export_settings)
-        theme_label = QLabel('Current Theme: ')
-        aspect_ratio_label = QLabel('Aspect Ratio: ')
-        transformation_label = QLabel('Image Transform: ')
-        line_wrap_label = QLabel('Line Wrap: ')
-        self.theme_dropdown = QComboBox(self, activated=self.set_theme)
-        self.aspect_ratio_dropdown = QComboBox(self, activated=self.set_aspect_ratio_method)
-        self.transformation_dropdown = QComboBox(self, activated=self.set_transformation_method)
-        self.line_wrap_dropdown = QComboBox(self, activated=self.set_line_wrap_method)
-        open_editor_button = QPushButton('Open Settings in Editor', clicked=self.open_editor)
-        key_show_button = QPushButton('Edit Auth Key', clicked=self.show_key)
-        key_copy_button = QPushButton('Copy to Clipboard', clicked=self.copy_key)
-        self.key_set_button = QPushButton('Set', clicked=self.set_key)
-        self.key_field = QLineEdit(returnPressed=self.key_set_button.click)
+        self.save_button:             QPushButton = QPushButton('Save', clicked=self.save_settings)
+        reload_button:                QPushButton = QPushButton('Reload', clicked=self.reload_settings)
+        import_button:                QPushButton = QPushButton('Import Settings', clicked=self.import_settings)
+        export_button:                QPushButton = QPushButton('Export Settings', clicked=self.export_settings)
+        theme_label:                  QLabel = QLabel('Current Theme: ')
+        aspect_ratio_label:           QLabel = QLabel('Aspect Ratio: ')
+        transformation_label:         QLabel = QLabel('Image Transform: ')
+        line_wrap_label:              QLabel = QLabel('Line Wrap: ')
+        self.theme_dropdown:          QComboBox = QComboBox(self)
+        self.aspect_ratio_dropdown:   QComboBox = QComboBox(self)
+        self.transformation_dropdown: QComboBox = QComboBox(self)
+        self.line_wrap_dropdown:      QComboBox = QComboBox(self)
+        open_editor_button:           QPushButton = QPushButton('Open Settings in Editor', clicked=self.open_editor)
+        key_show_button:              QPushButton = QPushButton('Edit Auth Key', clicked=self.show_key)
+        key_copy_button:              QPushButton = QPushButton('Copy to Clipboard', clicked=self.copy_key)
+        self.key_set_button:          QPushButton = QPushButton('Set', clicked=self.set_key)
+        self.key_field:               QLineEdit = QLineEdit(returnPressed=self.key_set_button.click)
+
+        self.theme_dropdown.activated.connect(self.set_theme)
+        self.transformation_dropdown.activated.connect(self.set_transformation_method)
+        self.line_wrap_dropdown.activated.connect(self.set_line_wrap_method)
+        self.aspect_ratio_dropdown.activated.connect(self.set_aspect_ratio_method)
 
         # Define layouts
         layout = QGridLayout()  # Main layout
@@ -82,9 +90,9 @@ class SettingsWindow(QWidget):
 
         # Assign positions of layouts
         self.setLayout(layout)
-        layout.addLayout(top, 0, 0, Qt.AlignmentFlag.AlignTop)
-        layout.addLayout(middle, 10, 0, Qt.AlignmentFlag.AlignTop)
-        layout.addLayout(bottom, 20, 0, Qt.AlignmentFlag.AlignBottom)
+        layout.addLayout(top, 0, 0, Qt.AlignTop)
+        layout.addLayout(middle, 10, 0, Qt.AlignTop)
+        layout.addLayout(bottom, 20, 0, Qt.AlignBottom)
 
         # Add top widgets
         top.addWidget(self.save_button)
@@ -115,7 +123,7 @@ class SettingsWindow(QWidget):
         # Modify properties of widgets
         theme_label.setMaximumWidth(85)
 
-        self.theme_dropdown.addItems(theme.display_name for theme in self.getter_window.APP.sorted_themes)
+        self.theme_dropdown.addItems([theme.display_name for theme in self.getter_window.APP.sorted_themes])
         self.theme_dropdown.setCurrentIndex(self.getter_window.APP.theme_index_map[self.settings['gui/themes/selected']])
 
         aspect_ratio_label.setMaximumWidth(70)
@@ -131,7 +139,7 @@ class SettingsWindow(QWidget):
         self.save_button.setDisabled(True)
         reload_button.setMaximumWidth(60)
 
-        self.key_field.setAlignment(Qt.AlignmentFlag.AlignCenter)
+        self.key_field.setAlignment(Qt.AlignCenter)
         self.key_field.setMinimumWidth(220)
         self.key_field.setFont(QFont('segoe ui', 8))
         self.key_set_button.setMinimumWidth(40)
@@ -200,14 +208,15 @@ class SettingsWindow(QWidget):
         self.key_set_button.setDisabled(True)
         self.key_field.setDisabled(True)
         self.key_field.setText(self.hidden_key())
-        self.key_field.setAlignment(Qt.AlignmentFlag.AlignCenter)
+        self.key_field.setAlignment(Qt.AlignCenter)
 
     def show_key(self) -> None:
         """Toggle hiding and showing the API key."""
         if not self.key_field.isEnabled():
-            self.key_field.setAlignment(Qt.AlignmentFlag.AlignLeft)
+            self.key_field.setAlignment(Qt.AlignLeft)
             self.key_field.setText(self.client.auth_key)
             self.key_field.setDisabled(False)
+            self.key_field.setFocus()
             self.key_set_button.setDisabled(False)
         else:
             self.hide_key()
@@ -219,8 +228,8 @@ class SettingsWindow(QWidget):
 
     def set_key(self) -> None:
         """Set the client's auth_key to the current text in the key field."""
-        self.save_button.setDisabled(False)
-        self.client.auth_key = self.key_field.text()
+        self.client.auth_key = self.key_field.text().strip().removeprefix('x-343-authorization-spartan: ')
+        self.show_key()
 
     def hidden_key(self) -> str:
         """:return: The first 5 and last 4 characters of the API key, seperated by periods."""
@@ -235,6 +244,7 @@ class SettingsWindow(QWidget):
         """Auto hides the key upon un-minimizing."""
         super().showEvent(event)
         self.key_set_button.setDisabled(True)
+        self.key_field.setAlignment(Qt.AlignCenter)
         self.key_field.setDisabled(True)
         self.key_field.setText(self.hidden_key())
 
@@ -248,32 +258,32 @@ class AppWindow(QMainWindow):
     def __init__(self, client: Client, app: GetterApp, size: QSize) -> None:
         """Create the window for the application."""
         super().__init__()
-        self.APP: GetterApp = app
-        self.client = client
-        self.clipboard: QClipboard | None = app.clipboard()
-        self.detached: dict[str, QMainWindow | None] = {'media': None, 'text': None}
+        self.APP:                   GetterApp = app
+        self.client:                Client = client
+        self.clipboard:             QClipboard = app.clipboard()
+        self.current_image:         QPixmap | None = None
+        self._clicked_input_field:  bool = False
+        self.detached:              dict[str, QMainWindow | None] = {'media': None, 'text': None}
         # self.themes: dict[str, str] = themes
         self.setWindowTitle(f'HaloInfiniteGetter v{__version__}')
         self.setWindowIcon(QIcon(str(RESOURCE_PATH / 'icons/hi.ico')))
         self.resize(size)
 
-        self.settings_window = SettingsWindow(self, QSize(420, 600))
-
         self.setContextMenuPolicy(Qt.ContextMenuPolicy.CustomContextMenu)
 
-        self._clicked_input_field: bool = False
-        self.current_image: QPixmap | None = None
         self.input_field:   QLineEdit
         self.media_output:  QGraphicsView
-        self.text_output:   QTextEdit
+        self.text_output:   QTextBrowser
         self.clear_picture: QPushButton
         self.copy_picture:  QPushButton
-        self.clear_text: QPushButton
-        self.copy_text:  QPushButton
+        self.clear_text:    QPushButton
+        self.copy_text:     QPushButton
 
         self._init_toolbar()
         self._init_ui()
 
+        self.settings_window = SettingsWindow(self, QSize(420, 600))
+
     def _init_toolbar(self) -> None:
         """Initialize toolbar widgets."""
         self.toolbar = QToolBar('Toolbar', self)
@@ -294,20 +304,21 @@ class AppWindow(QMainWindow):
         """Initialize the UI, including Layouts and widgets."""
         self.media_frame = QFrame()
         self.image_size_label = QLabel('Image Output:')
-        self.image_detach_button = QPushButton('Detach', clicked=self.media_detach_handler)
+        self.image_detach_button = QPushButton('Detach', clicked=self.toggle_media_detach)
         self.media_output = QGraphicsView()
 
         self.text_frame = QFrame()
         self.text_size_label = QLabel('Text Output:')
-        self.text_detach_button = QPushButton('Detach', clicked=self.text_detach_handler)
-        self.text_output = QTextEdit()
+        self.text_detach_button = QPushButton('Detach', clicked=self.toggle_text_detach)
+        self.text_output = QTextBrowser()
+        self.text_output.anchorClicked.connect(lambda e: self.navigate_to(e.toDisplayString()))
 
         self.clear_picture = QPushButton('Clear', clicked=self.clear_current_pixmap)
         self.copy_picture = QPushButton('Copy Picture', clicked=self.copy_current_pixmap)
         self.clear_text = QPushButton('Clear', clicked=self.clear_current_text)
         self.copy_text = QPushButton('Copy Text', clicked=self.copy_current_text)
 
-        subdomain_field = QLineEdit(self.client._sub_host)
+        subdomain_field = QLineEdit(self.client.sub_host)
         root_folder_field = QLineEdit(self.client.PARENT_PATH)
         get_button = QPushButton('GET', clicked=self.get_resource)
         scan_button = QPushButton('SCAN', clicked=self.scan_resource)
@@ -328,9 +339,9 @@ class AppWindow(QMainWindow):
 
         self.setCentralWidget(main_widget)
         main_widget.setLayout(layout)
-        layout.addLayout(top, 0, 0, Qt.AlignmentFlag.AlignTop)
-        layout.addLayout(self.outputs, 10, 0, Qt.AlignmentFlag.AlignHCenter)
-        layout.addLayout(bottom, 20, 0, Qt.AlignmentFlag.AlignBottom)
+        layout.addLayout(top, 0, 0, Qt.AlignTop)
+        layout.addLayout(self.outputs, 10, 0, Qt.AlignHCenter)
+        layout.addLayout(bottom, 20, 0, Qt.AlignBottom)
 
         top.addWidget(subdomain_field)
         top.addWidget(root_folder_field)
@@ -342,23 +353,25 @@ class AppWindow(QMainWindow):
         self.outputs.addWidget(self.media_frame)
         self.outputs.addWidget(self.text_frame)
 
+        # noinspection Duplicates
         self.media_frame.setLayout(media_layout)
         media_layout.addLayout(media_top)
         media_layout.addWidget(self.media_output)
         media_layout.addLayout(media_bottom)
-        media_top.addWidget(self.image_size_label, Qt.AlignmentFlag.AlignLeft)
-        media_top.addWidget(self.image_detach_button, Qt.AlignmentFlag.AlignRight)
-        media_bottom.addWidget(self.clear_picture, Qt.AlignmentFlag.AlignLeft)
-        media_bottom.addWidget(self.copy_picture, Qt.AlignmentFlag.AlignLeft)
+        media_top.addWidget(self.image_size_label, Qt.AlignLeft)
+        media_top.addWidget(self.image_detach_button, Qt.AlignRight)
+        media_bottom.addWidget(self.clear_picture, Qt.AlignLeft)
+        media_bottom.addWidget(self.copy_picture, Qt.AlignLeft)
 
+        # noinspection Duplicates
         self.text_frame.setLayout(text_layout)
         text_layout.addLayout(text_top)
         text_layout.addWidget(self.text_output)
         text_layout.addLayout(text_bottom)
-        text_top.addWidget(self.text_size_label, Qt.AlignmentFlag.AlignLeft)
-        text_top.addWidget(self.text_detach_button, Qt.AlignmentFlag.AlignRight)
-        text_bottom.addWidget(self.clear_text, Qt.AlignmentFlag.AlignLeft)
-        text_bottom.addWidget(self.copy_text, Qt.AlignmentFlag.AlignLeft)
+        text_top.addWidget(self.text_size_label, Qt.AlignLeft)
+        text_top.addWidget(self.text_detach_button, Qt.AlignRight)
+        text_bottom.addWidget(self.clear_text, Qt.AlignLeft)
+        text_bottom.addWidget(self.copy_text, Qt.AlignLeft)
         text_bottom.setSpacing(5)
 
         subdomain_field.setFixedWidth(125)
@@ -366,7 +379,7 @@ class AppWindow(QMainWindow):
         root_folder_field.setFixedWidth(28)
         root_folder_field.setDisabled(True)
         self.input_field.mousePressEvent = self._destroy_text_on_first_click(self.input_field.mousePressEvent)
-        # subdomainField.returnPressed.connect(lambda *_: self.client.__class__.host.fset(self.client, subdomainField.text()))
+        # subdomain_field.returnPressed.connect(lambda *_: self.client.__class__.host.fset(self.client, subdomain_field.text()))
         get_button.setMaximumWidth(40)
         scan_button.setMaximumWidth(55)
         self.image_size_label.setMinimumWidth(50)
@@ -381,6 +394,7 @@ class AppWindow(QMainWindow):
         self.text_detach_button.setMaximumWidth(80)
         self.text_output.setMinimumHeight(28)
         self.text_output.setLineWrapMode(QTextEdit.LineWrapMode(self.APP.settings['gui/text_output/line_wrap_mode']))
+        self.text_output.setOpenLinks(False)
         self.text_output.setDisabled(True)
         self.clear_picture.setMaximumWidth(80)
         self.clear_picture.setMinimumWidth(40)
@@ -415,7 +429,7 @@ class AppWindow(QMainWindow):
         window = QMainWindow(self)
         window.setWindowTitle(title if title is not None else self.windowTitle())
         window.setCentralWidget(frame)
-        window.closeEvent = lambda *_: handler()
+        window.closeEvent = lambda *_: handler() if self.detached['media'] is not None else None
         window.setMinimumHeight(200)
         window.setMinimumWidth(300)
         return window
@@ -438,37 +452,43 @@ class AppWindow(QMainWindow):
         self.settings_window.activateWindow()
         self.settings_window.raise_()
 
-    def media_detach_handler(self) -> None:
-        """Handler for detaching and reattaching the medio output."""
+    def navigate_to(self, path: str) -> None:
+        """Set input field text to path and get resource."""
+        self.input_field.setText(path)
+        self.get_resource()
+
+    def toggle_media_detach(self) -> None:
+        """Handler for detaching and reattaching the media output."""
         if self.detached['media'] is None:
-            self.detached['media'] = window = self._setup_detached_window(self.media_frame, self.media_detach_handler, 'Detached Image Output')
+            self.detached['media'] = window = self._setup_detached_window(self.media_frame, self.toggle_media_detach, 'Detached Image Output')
             self.image_detach_button.setText('Reattach')
             window.resizeEvent = lambda *_: self.resize_image()
             window.show()
         else:
             window = self.detached['media']
             self.detached['media'] = None
-            window.destroy()
+            window.close()
 
             self.outputs.insertWidget(0, self.media_frame)
             self.image_detach_button.setText('Detach')
 
-    def text_detach_handler(self) -> None:
+    def toggle_text_detach(self) -> None:
         """Handler for detaching and reattaching the text output."""
         if self.detached['text'] is None:
-            self.detached['text'] = window = self._setup_detached_window(self.text_frame, self.text_detach_handler, 'Detached Text Output')
+            self.detached['text'] = window = self._setup_detached_window(self.text_frame, self.toggle_text_detach, 'Detached Text Output')
             self.text_detach_button.setText('Reattach')
             window.show()
         else:
             window = self.detached['text']
             self.detached['text'] = None
-            window.destroy()
+            window.close()
 
             self.outputs.insertWidget(-1, self.text_frame)
             self.text_detach_button.setText('Detach')
 
     def clear_current_pixmap(self) -> None:
         """Clear the current image from the media output."""
+        self.image_size_label.setText('Image Output: ')
         self.clear_picture.setDisabled(True)
         self.copy_picture.setDisabled(True)
         self.media_output.scene().clear()
@@ -481,6 +501,7 @@ class AppWindow(QMainWindow):
 
     def clear_current_text(self) -> None:
         """Clear the current text from the text output."""
+        self.text_size_label.setText('Text Output: ')
         self.clear_text.setDisabled(True)
         self.copy_text.setDisabled(True)
         self.text_output.setDisabled(True)
@@ -527,15 +548,28 @@ class AppWindow(QMainWindow):
                     self.image_size_label.setText(f'Image Output: {size.width()}x{size.height()} ({round(len(data) / 1024, 4)} KiB)')
                     self.resize_image()
                 else:
-                    if isinstance(data, dict):
-                        data = json.dumps(data, indent=2)
-
-                    data = str(data)
                     self.clear_text.setDisabled(False)
                     self.copy_text.setDisabled(False)
                     self.text_output.setDisabled(False)
-                    self.text_output.setText(data)
-                    self.text_size_label.setText(f'Text Output: {len(data.splitlines())} lines {len(data)} characters '
+
+                    if isinstance(data, dict):
+                        data = json.dumps(data, indent=2)
+                    elif isinstance(data, int):
+                        data = f'{data}: {HTTP_CODE_MAP[data][0]}\n{HTTP_CODE_MAP[data][1]}'
+
+                    output = data
+                    replaced = set()
+
+                    for match in PATH_PATTERN.finditer(data):
+                        match = match[0].replace('"', '')
+                        if match not in replaced:
+                            output = output.replace(match, f'<a href="{match}" style="color: #2A5DB0">{match}</a>')
+                            replaced.add(match)
+
+                    self.text_output.setHtml(f'<body style="white-space: pre-wrap">{output}</body>')
+                    self.text_size_label.setText(f'Text Output: '
+                                                 f'{len(data.splitlines())} lines; '
+                                                 f'{len(data)} characters '
                                                  f'({round(len(data.encode("utf8")) / 1024, 4)} KiB)')
             elif op_code == 20:
                 self.client.recursive_search(user_input)
@@ -556,26 +590,24 @@ class AppWindow(QMainWindow):
             self.media_output.scene().addPixmap(new)
 
     # # # # # Events
+    def show(self) -> None:
+        """After window is displayed, show warnings if not already warned."""
+        super().show()
+        if not self.shown_key_warning and self.client.auth_key is None:
+            QMessageBox.warning(
+                self,
+                'Empty API Token', '''
+The spartan authorization token is not set, please set SPARTAN_AUTH environment variable and restart, or set the auth key value in Settings.
+
+You will be unable to acquire new data until a new token is provided.''')
+            self.__class__.shown_key_warning = True
 
     def resizeEvent(self, event: QResizeEvent) -> None:
         """Resize image on resize of window."""
         super().resizeEvent(event)
         self.resize_image()
 
-    def showEvent(self, event: QShowEvent) -> None:
-        """Shows warnings on startup."""
-        super().showEvent(event)
-        if not self.shown_key_warning and self.client.auth_key is None:
-            self.__class__.shown_key_warning = True
-            QMessageBox.warning(
-                self,
-                'Empty API Token', '''
-The SPARTAN_AUTH environment variable is not set!
-You will be unable to acquire new data until a new token is provided.
-
-You can manually set it in the Settings window.''')
-
     def closeEvent(self, event: QCloseEvent) -> None:
-        """Closes all detached/children windows."""
+        """Closes all detached/children windows and quit application."""
         super().closeEvent(event)
-        self.APP.closeAllWindows()
+        self.APP.quit()
